(function () {
  // 如果window对象上存在WebViewJavascriptBridge直接返回，避免重复初始化
  if (window.WebViewJavascriptBridge) {
    return;
  }

  var detect = {
    isIOS: /iphone|ipod|ipad/gi.test(navigator.userAgent),
    isAndroid: navigator.userAgent.toLowerCase().indexOf('android') > -1
  };
  // callNative回调对象
  var responseCallbacks = {};
  // callJS消息handlers
  var callJSMessageHandlers = {};
  // callback唯一值
  var uniqueId = 1;

  // 解析json字符串
  function parseJson(jsonString, log) {
    var data = null;
    if (typeof jsonString === 'string') {
      try {
        data = JSON.parse(jsonString);
      } catch (e) {
        console.log(log || 'json解析失败');
      }
    }
    return data;
  }

  // 调用原生方法，屏蔽两端差异
  function _sendMessage(message) {
    // 消息需要转成字符串
    var strMsg = '';
    if (typeof message === 'object') {
      strMsg = JSON.stringify(message);
    }
    if (detect.isIOS) {
      // ios端通过window.webkit.messageHandlers.xxxx.postMessage调用，xxxx可自定义
      window.webkit.messageHandlers.wyHandlers.postMessage(strMsg);
    } else if (detect.isAndroid) {
      // android端通过window.xxxx.yyyy调用，xxxx和yyyy均可自定义
      window.wyHandlers.postMessage(strMsg);
    }
  }

  // 创建通信消息
  function _createMessage(method, data, callback) {
    var callbackId = 'wybridge_cb_' + (uniqueId++) + Date.now();
    responseCallbacks[callbackId] = callback;
    return {
      handlerName: method,
      data: data,
      callbackId: callbackId
    }
  }

  // 生成原生回调函数
  function _createResponseFunc (handlerName, callbackId) {
    return function (responseData) {
      if (!callbackId) {
        console.log('no callback');
        return;
      }
      // 原生调用js方法回调同样通过sendMessage实现
      _sendMessage({
        handlerName: handlerName,
        responseId: callbackId,
        responseData: responseData
      });
    }
  }

  // 调用原生方法
  function _callNative(method, data, callback) {
    var message = _createMessage(method, data, callback);

    _sendMessage(message);
  }

  // 原生调用JS方法
  function _callJS(jsonString) {
    var message = parseJson(jsonString) || {};
    var handlerName = message.handlerName;
    var data = message.data;
    var callbackId = message.callbackId;
    // 原生方法回调
    var responseCallback = _createResponseFunc(handlerName, callbackId);
    // 判断注册方法是否存在
    if (callJSMessageHandlers[handlerName]) {
      callJSMessageHandlers[handlerName](data, responseCallback);
    } else {
      responseCallback({
        code: 404,
        message: '接口不存在'
      });
    }
  }

  // 调用原生方法回调
  function _nativeCallback(jsonString) {
    var data = parseJson(jsonString) || {};
    // 获取响应id
    var responseId = data.responseId;
    // 不存在响应id，直接返回
    if (!responseId) {
      console.log('responseId is not found');
      return;
    }
    // 删除对象上responseId
    delete data.responseId;
    var cb = responseCallbacks[responseId];
    if (typeof cb !== 'function') {
      return;
    }
    // 兼容老版本
    cb(data.responseData);
    delete responseCallbacks[responseId];
  }

  // 提供给native层调用的JS方法
  window._nativeBridge = {
    callJS: _callJS,
    nativeCallback: _nativeCallback
  };

  // 提供给前端注册给native调用的方法
  function registerHandler(method, callback) {
    callJSMessageHandlers[method] = callback;
  }

  // 提供给前端调用native的方法
  function callHandler(method, data, callback) {
    if (arguments.length === 2 && typeof data === 'function') {
      callback = data;
      data = null;
    }
    if (typeof callback !== 'function') {
      callback = function () {};
    }
    _callNative(method, data, callback);
  }

  // 兼容老版本提供的init方法
  function init() {}

  // 提供给前端的bridge对象
  // 兼容老版本，保留init方法
  var wyBridge = {
    init: init,
    callHandler: callHandler,
    registerHandler: registerHandler
  };

  window.WebViewJavascriptBridge = wyBridge;
  // 兼容老版本事件下发
  var event = document.createEvent('Event');
  event.initEvent('WebViewJavascriptBridgeReady');
  document.dispatchEvent(event);

  if (detect.isIOS) {
    // ios端通过window.webkit.messageHandlers.xxxx.postMessage调用，xxxx可自定义
    window.webkit.messageHandlers.wyHandlers.webViewJavascriptBridgeReady();
  } else if (detect.isAndroid) {
    // android端通过window.xxxx.yyyy调用，xxxx和yyyy均可自定义
    window.wyHandlers.webViewJavascriptBridgeReady();
  }

  // 兼容老版本事件下发，目前仅ios端需要
  if (Array.isArray(window.WVJBCallbacks)) {
    window.WVJBCallbacks.forEach(function (cb) {
      typeof cb === 'function' && cb(wyBridge);
    });
  }
})();
